Juju charm apache
=================

The Apache Software Foundation's goal is to build a secure, efficient
and extensible HTTP server as standards-compliant open source
software. The result has long been the number one web server on the
Internet.  It features support for HTTPS, virtual hosting, CGI, SSI,
IPv6, easy scripting and database integration, request/response
filtering, many flexible authentication schemes, and more.

How to deploy the charm
-----------------------
    juju deploy apache2
    juju set apache2 "vhost_http_template=$(base64 < vhost.tmpl)"
    # and / or
    juju set apache2 "vhost_https_template=$(base64 < vhost.tmpl)"
    juju add-relation apache2:reverseproxy haproxy:website

Vhost templates
---------------
The charm expects a jinja2 template to be passed in. The variables in
the template should relate to the services that apache will be proxying
-- obviously no variables need to be specified if no proxying is needed.

The charm will create the service variable, with the unit_name, when
the reverseproxy relationship is joined and present this to the template
at which point the vhost will be generated from the template again.  
All config settings are also available to the template.

For example to access squid then the {{ squid }} variable should be used.
This will be populated with the hostname:port of the squid service. The
individual hostname and port can also be accessed via squid_hostname and
squid_port.

Note: The service name should be used, not the charm name.  If deploying
      a charm with a different service name, use that instaed.

The joining charm may also set an all_services variable which contains
a list of services it provides in yaml format (list of associative arrays):

    # ... in haproxy charm, website-relation-joined
    relation-set all_services="
      - {service_name: gunicorn, service_port: 80}
      - {service_name: solr, service_port: 8080}
      - {service_name: my-webapp, service_port: 9090}
    "

then variables for each service would be available to the jinja2 template
in <juju_service_name>_<sub_service_name>.  In our example above 
haproxy contains stanzas named gunicorn, solr and my-webapp.  These are
accessed as {{ haproxy_gunicorn }}, {{ haproxy_solr }} and
{{ haproxy_mywebapp }} respectively.  If any unsupported characters are in
your juju service name or the service names exposed through "all_services",
they will be stripped.

For example a vhost that will pass all traffic on to an haproxy instance:

    <VirtualHost *:80>
    	ServerName radiotiptop.org.uk
    
    	CustomLog /var/log/apache2/radiotiptop-access.log combined
    	ErrorLog /var/log/apache2/radiotiptop-error.log
    
        DocumentRoot /srv/radiotiptop/www/root
    
        ProxyRequests off
        <Proxy *>
            Order Allow,Deny
            Allow from All
            ErrorDocument 403 /offline.html
            ErrorDocument 500 /offline.html
            ErrorDocument 502 /offline.html
            ErrorDocument 503 /offline.html
        </Proxy>
    
        ProxyPreserveHost off
        ProxyPassReverse / http://{{ haproxy_gunicorn }}/
    
        RewriteEngine on
    
        RewriteRule ^/$ /index.html [L]
        RewriteRule ^/(.*)$ http://{{ haproxy_gunicorn }}/$1 [P,L]
    </VirtualHost>

Certs, keys and chains
----------------------
ssl_keylocation, ssl_certlocation and ssl_chainlocation are file names in the 
charm /data directory.  If found, they will be copied as follows:

  - /etc/ssl/private/<ssl_keylocation>
  - /etc/ssl/certs/<ssl_certlocation>
  - /etc/ssl/certs/<ssl_chainlocation>

ssl_key and ssl_cert can also be specified which are are assumed to be
base64 encoded.  If specified, they will be written to appropriate directories
given the values in ssl_keylocation and ssl_certlocation as listed above.

ssl_cert may also be set to SELFSIGNED, which will generate a certificate.
This, of course, is mostly useful for testing and staging purposes.  The
generated certifcate/key will be placed according to ssl_certlocation and
ssl_keylocation as listed above.

{enable,disable}_modules
------------------------
Space separated list of modules to be enabled or disabled. If a module to
be enabled cannot be found then the charm will attempt to install it.

TODO:
-----

  * Document the use of balancer, nrpe, logging and website-cache
  * Method to deliver site content. This maybe by converting the charm to a
    subordinate and making it the master charms problem
  * Implement secure method for delivering key.  Juju will likely need to provide
    this.
  * Tuning. No tuning options are present. Convert apache2.conf to a template
    and expose config options
  * Testing. I have only tested the relationship setup with 1 haproxy instance.
    Needs testing against multiple instances 
